precision highp float;

varying vec2 v_TexPosition;
varying vec2 v_FragmentPosition;

uniform sampler2D u_Texture2D;
uniform sampler2D u_Mask;
uniform sampler2D u_Pattern;
uniform float u_MaxRadius;
uniform vec4 u_FillColor;
uniform vec2 u_StartPos;
uniform float u_Hardness;
uniform vec4 u_BaseColor;
uniform float u_Opacity;
uniform float u_Tolerance;
uniform vec2 u_PatternScale;
uniform vec2 u_PatternOffset;
uniform int u_TileFill;

void main() {
	float d = distance(v_FragmentPosition, u_StartPos);
	float a = clamp(d / u_MaxRadius, 0.0, 1.0);

	// pow(0.0, 0.0) results to NaN
	if (u_Hardness > 0.999) {
		a = 1.0;
	} else {
		a = pow(1.0 - a, mix(3.0, 0.0, u_Hardness));
	}

	vec4 color = texture2D(u_Texture2D, v_TexPosition);
	float p = texture2D(u_Mask, v_TexPosition).r * a;
	vec4 diffVector = u_BaseColor - color;

	float colorDiff = max(abs(diffVector.r), max(abs(diffVector.g), abs(diffVector.b)));
	float alphaDiff = abs(diffVector.a);

	vec3 filledColor = mix(color.rgb,  u_FillColor.rgb, p * (1.0 - colorDiff));
	float filledAlpha = mix(0.0, u_Opacity, (1.0 - colorDiff) * (1.0 - alphaDiff));

	vec4 srcColor = vec4(filledColor, filledAlpha);
	vec4 dstColor = color;

	vec4 resColor;
	resColor.rgb = mix(dstColor.rgb, srcColor.rgb, u_Opacity);
	resColor.a = srcColor.a + dstColor.a * (1.0 - srcColor.a);
	if (u_TileFill == 1) {
		vec2 patternPosition = v_TexPosition * u_PatternScale + u_PatternOffset;
    	vec4 patternColor = texture2D(u_Pattern, patternPosition);
    	gl_FragColor = mix(color, mix(color, resColor, patternColor.a), p);
	} else {
		gl_FragColor = mix(color, resColor, p);
	}
}