#define PI 3.1415926535897932384626433832795

precision highp float;

uniform mat3 u_Matrix;

uniform float u_Sigma;

attribute vec2 a_Position;
attribute vec2 a_TextureCoordinate;

varying float v_WeightArray[3];
varying float v_OffsetArray[2];

varying float v_WeightSum;

varying vec2 v_TextureCoordinate;

float computeWeight(float x);

void main() {
	float w0 = computeWeight(0.0);
	float w1 = computeWeight(1.0);
	float w2 = computeWeight(2.0);
	float w3 = computeWeight(3.0);
	float w4 = computeWeight(4.0);

	v_WeightArray[0] = w0;
	v_WeightArray[1] = w1 + w2;
	v_WeightArray[2] = w3 + w4;

	v_OffsetArray[0] = (1.0 * w1 + 2.0 * w2) / (w1 + w2);
	v_OffsetArray[1] = (3.0 * w3 + 4.0 * w4) / (w3 + w4);

	v_WeightSum = w0 + w1 + w2 + w3 + w4;

	v_TextureCoordinate = a_TextureCoordinate;

	gl_Position = vec4(u_Matrix * vec3(a_Position, 1.0), 1.0);
}

float computeWeight(float x) {
	float sigmaSqrDbl = 2.0 * u_Sigma * u_Sigma;

	return (1.0 / (sqrt(PI * sigmaSqrDbl))) * exp(- x * x / sigmaSqrDbl);
}